/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package firealarmjava;

import java.io.IOException;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.lang.Math;
import java.util.Calendar;
import java.util.Date;

import net.tinyos.message.Message;
import net.tinyos.message.MessageListener;
import net.tinyos.message.MoteIF;
import net.tinyos.packet.BuildSource;
import net.tinyos.packet.PhoenixSource;
import net.tinyos.util.PrintStreamMessenger;

/**
 *
 * @author Sebastia
 */
public class FireAlarmJava implements MessageListener {
    
    //Constants declaration 
    public static final int AM_BROADCAST_ADDR = 0xFFFF;
    public static final int VREF_MILIVOLTS = 2560;
    
    //Variables declaration
    public static FireAlarmJava instance = null;
    static ControlConsole consola;
    private MoteIF mif_rx;
    static MoteIF mif_tx;
    static Message payload;
    static ArrayList<Sensor> sensors = new ArrayList<Sensor>();

    //End of variables declaration

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        
        consola = new ControlConsole();
        instance = new FireAlarmJava();
        consola.launchFrame();
	
        PhoenixSource phoenix;
        //phoenix = BuildSource.makePhoenix(PrintStreamMessenger.err);

        String source="sf@localhost:9002";	

        phoenix = BuildSource.makePhoenix(source, PrintStreamMessenger.err);

        instance.mif_rx = new MoteIF(phoenix);
        instance.mif_tx = new MoteIF(phoenix);
        
        instance.mif_rx.registerListener(new BatAlarmMsg(), instance);
        instance.mif_rx.registerListener(new BatMsg(), instance);
        instance.mif_rx.registerListener(new DataMsg(), instance);
        instance.mif_rx.registerListener(new FireAlarmMsg(), instance);
        instance.mif_rx.registerListener(new StateReqMsg(), instance);
        instance.mif_rx.registerListener(new TempMsg(), instance);
        
        
    }
    
    // Returns the number of sensors
    public int get_NumSensors(){
        return sensors.size();
    }

    private static void println(String s){
            System.out.println(s);
    }

    @Override
    public void messageReceived(int to, Message m) {
		
        if(m instanceof DataMsg){
            DataMsg dm = (DataMsg) m;
            Sensor sensor = new Sensor(dm.get_moteId());
            //Create a new sensor if it doesn't exist
            int index = indexSensor(sensor);
            sensor = (Sensor) sensors.get(index);
            //Create a register message
            SensorMessage msg_rx = new SensorMessage(dm.toString(),true);
            
            sensor.setVref(dm.get_vrefmilivolts());
            sensor.setActive(true);
            sensor.setCurrentTemp(dm.get_countsTemp());
            sensor.setCurrentBatLevel(dm.get_countsBat());
            sensor.setCurrentLight(dm.get_countsPhoto());
            sensor.setTempThd(dm.get_tempThd());
            sensor.setTempTimerPeriod(dm.get_tempTimer());
            sensor.setBatTimerPeriod(dm.get_batTimer());
            sensor.setDebugTimerPeriod(dm.get_debugTimer());
            sensor.AddMessage(msg_rx);
            sensor.setAllData(true);
            sensors.set(index, sensor);
            consola.actualAll();
            refreshConsole(sensor);
            consola.refreshColors(2);
            
            String temp = toCentig(sensor.getCurrentTemp(), sensor.getVref());
            String bat = toVolts(sensor.getCurrentBatLevel(),sensor.getVref());
            StringBuilder sb = new StringBuilder();
            sb.append("DATA REPORT" + "\n\r");
            sb.append("Mote Id: " + dm.get_moteId() + "\n");
            sb.append("Temperature: " + temp + " ºC" + "\n");
            sb.append("Battery level: "+ bat + " V" + "\n");
            println(sb.toString());
        }
        
        //Temperature Message received
        if(m instanceof TempMsg){
            TempMsg tm = (TempMsg) m;
            Sensor sensor = new Sensor(tm.get_moteId());
            
            //Create a register message
            SensorMessage msg_rx = new SensorMessage(tm.toString(),true);
            //Create a new sensor if it doesn't exist
            int index = indexSensor(sensor);
            sensor.setVref(tm.get_vrefmilivolts());
            sensor.setActive(true);
            sensor.setCurrentTemp(tm.get_countsTemp());
            sensor.AddMessage(msg_rx);
            sensors.set(index, sensor);
            refreshConsole(sensor);
            
            String temp = toCentig(sensor.getCurrentTemp(), sensor.getVref());
            println("Temperature: "+ temp + " ºC" + "\n");
        }
        
        //Battery Level Message received
        if(m instanceof BatMsg){
            BatMsg bm = (BatMsg) m;
            Sensor sensor = new Sensor(bm.get_moteId());
            //Create a new sensor if it doesn't exist
            int index = indexSensor(sensor);
            sensor = (Sensor) sensors.get(index);
            //Create a register message
            SensorMessage msg_rx = new SensorMessage(bm.toString(),true);

            sensor.setVref(bm.get_vrefmilivolts());
            sensor.setActive(true);
            sensor.setCurrentBatLevel(bm.get_countsBat());
            sensor.AddMessage(msg_rx);
            sensors.set(index, sensor);
            refreshConsole(sensor);
            consola.refreshColors(1);
            
            String bat = toVolts(sensor.getCurrentBatLevel(),sensor.getVref());
            println("Battery level: "+ bat + " V" + "\n");
        }
        
        //Battery Alarm Message received
        if(m instanceof BatAlarmMsg){
            BatAlarmMsg bam = (BatAlarmMsg) m;
            Sensor sensor = new Sensor(bam.get_moteId());

            //Create a new sensor if it doesn't exist
            int index = indexSensor(sensor);
            sensor = (Sensor) sensors.get(index);
            //Create a register message
            SensorMessage msg_rx = new SensorMessage(bam.toString(),true);
            
            sensor.setVref(bam.get_vrefmilivolts());
            sensor.setActive(true);
            sensor.setCurrentBatLevel(bam.get_countsBat());
            if(!sensor.isBatAlarm())sensor.setBatDate(getCurrentTime());
            sensor.setBatAlarm(true);
            sensor.AddMessage(msg_rx);
            sensors.set(index, sensor);
            refreshConsole(sensor);
            consola.refreshColors(1);
            
            String bat = toVolts(sensor.getCurrentBatLevel(),sensor.getVref());
            println("Alarm: Low Battery: "+ bat + " V" + 
		"\n\r" + "Mote Id: " + bam.get_moteId() + "\n\r" +
			sensor.getBatDate());
        }
        
        //Fire Alarm Message received
        if(m instanceof FireAlarmMsg){
            FireAlarmMsg fam = (FireAlarmMsg) m;
            Sensor sensor = new Sensor(fam.get_moteId());
            //Create a new sensor if it doesn't exist
            int index = indexSensor(sensor);
            sensor = (Sensor) sensors.get(index);
            //Create a register message
            SensorMessage msg_rx = new SensorMessage(fam.toString(),true);
            
            sensor.setVref(fam.get_vrefmilivolts());
            sensor.setActive(true);
            sensor.setCurrentTemp(fam.get_countsTemp());
            sensor.setManualAlarm(fam.get_manualAlarm() == (byte)1);
            sensor.AddMessage(msg_rx);
            String temp = toCentig(sensor.getCurrentTemp(), sensor.getVref());

            if(!sensor.isFireAlarm()){
            	sensor.setFireDate(getCurrentTime());
            	try {Thread.sleep(100);}
        		catch (InterruptedException exception) {}
                sensor.setFireAlarm(true);
                //Create fire alarm Ack message
                FireAlarmAckMsg p_load = new FireAlarmAckMsg();
                p_load.set_ackAlarm((byte)1);
                payload = p_load;
                //Registers the message (false => Tx)
                SensorMessage msg_tx = new SensorMessage(p_load.toString(),false);
                sensor.AddMessage(msg_tx);
                //Send fire akc
                try {
                        mif_tx.send(fam.get_moteId(), payload);
                    }
                catch (IOException exception) {
                    System.err.println("Exception thrown when sending Ack packet. Exiting.");
                    System.err.println(exception);
                }
                //Fire Alarm Acknowledge Message sent
                println("Fire Alarm Acknowledge Message sent");
            }
            println("Alarm: Fire!!" + "\n\r" + "Mote Id: " + fam.get_moteId() +
            		"\n\r" + "Temperature: "+ temp + " ºC" + "\n\r" +
            		sensor.getFireDate());
            sensors.set(index, sensor);
            refreshConsole(sensor);   
        }
        
        //State Request Message received
        if(m instanceof StateReqMsg){
            StateReqMsg srm = (StateReqMsg) m;
            Sensor sensor = new Sensor(srm.get_moteId());
            //Create a new sensor if it doesn't exist
            int index = indexSensor(sensor);
            sensor = (Sensor) sensors.get(index);
            //Registers the message Rx => true
            SensorMessage msg_rx = new SensorMessage(srm.toString(),true);
            sensor.AddMessage(msg_rx);
            sensor.setAllData(false);
            
            //Configure state request response
            StateMsg p_load = new StateMsg();
            byte state = 0;
            if (sensor.isActive()) state = 1;
            p_load.set_stateOn(state);
            p_load.set_tempThd(sensor.getTempThd());
            p_load.set_tempTimerPeriod(sensor.getTempTimerPeriod());
            p_load.set_batTimerPeriod(sensor.getBatTimerPeriod());
            p_load.set_debugTimerPeriod(sensor.getDebugTimerPeriod());
            
            payload = p_load;
            //Registers the message Tx => false
            SensorMessage msg_tx = new SensorMessage(p_load.toString(),false);
            sensor.AddMessage(msg_tx);
            sensors.set(index, sensor);
            refreshConsole(sensor);
            println("State Request Message received");
            
            try {Thread.sleep(1000);}
    		catch (InterruptedException exception) {}

            //Send state message
            try {
                    mif_tx.send(srm.get_moteId(), payload);
                }
            catch (IOException exception) {
                System.err.println("Exception thrown when sending Ack packet. Exiting.");
                System.err.println(exception);
            }
            println("State Message sent");
//            startTimerData();
        }
    }
    
    /**
     * Procedure that return sensors index of sensor
     * @param sensor
     * @return
     */
    private int indexSensor(Sensor sensor){
    	int index = -1;
    	for(int i = 0; i < sensors.size(); i++){
    		if(sensors.get(i).getNodeId() == sensor.getNodeId())
    			index = i;
    	}
    	if(index == -1){
            sensors.add(sensor);
            consola.addSensor(sensor);
//            startTimerData();
            index = indexSensor(sensor);
    	}
    	return index;
    }
    
    /**
     * Getter Sensor nodeId == nId
     * @param nId
     * @return
     */
    public static Sensor getSensor(short nId){
    	int index = -1;
    	for(int i = 0; i < sensors.size(); i++){
    		if(sensors.get(i).getNodeId() == nId)
    			index = i;
    	}
    	if(!(index == -1))	return sensors.get(index);
    	else return new Sensor(nId);
    }
    
    /**
     * Procedure to refresh console
     * @param sensor
     */
    private static void refreshConsole(Sensor sensor){
    	if(consola.isCurrSens(sensor)){
    		consola.updateFrame(sensor);
    	}
    }
    
    /**
     * Procedure to send request data message
     * @param sensor
     */
    public static void requestData(Sensor sensor){
    	//Configure data request message
        DataReqMsg p_load = new DataReqMsg();
        p_load.set_seq((byte)0);
        payload = p_load;
        //Registers the message (false => Tx)
        SensorMessage msg_tx = new SensorMessage(p_load.toString(),false);
        sensor.AddMessage(msg_tx);
        sensor.setAllData(false);
        //Send data request message
        try {
                mif_tx.send(sensor.getNodeId(), payload);
            }
        catch (IOException exception) {
            System.err.println("Exception thrown when sending packet. Exiting.");
            System.err.println(exception);
        }
        println("Data Request Message sent");
    }
    
    /**
     * Procedure to set fire alarm temperature threshold
     * @param Temp
     */
    public static void setTempThreshold(double Temp){
            int tempThreshold = (int)(Temp*1024/VREF_MILIVOLTS);
            TempThdMsg p_load = new TempThdMsg();
            p_load.set_countsTemp(tempThreshold);
            payload = p_load;
            //Registers the message (false => Tx)
            SensorMessage msg_tx = new SensorMessage(p_load.toString(),false);
            Sensor sensor = getSensor(consola.getCurrentnodeId());
            sensor.AddMessage(msg_tx);
            //Send new temperature threshold message
            try {
                    mif_tx.send(sensor.getNodeId(), payload);
                }
            catch (IOException exception) {
                System.err.println("Exception thrown when sending packet. Exiting.");
                System.err.println(exception);
            }
            println("Set Temperature Threshhold message sent");
            try {Thread.sleep(500);}
    		catch (InterruptedException exception) {}
    		requestData(sensor);
    }

    /**
     * Procedure to set temperature timer period
     * @param teTi
     */
    public static void setTempTimer(int teTi){
            TempTimerMsg p_load = new TempTimerMsg();
            p_load.set_tempTimerPeriod(teTi);
            payload = p_load;
            //Registers the message (false => Tx)
            SensorMessage msg_tx = new SensorMessage(p_load.toString(),false);
            Sensor sensor = getSensor(consola.getCurrentnodeId());
            sensor.AddMessage(msg_tx);
            //Send new temperature timer period message
            try {
                    mif_tx.send(sensor.getNodeId(), payload);
                }
            catch (IOException exception) {
                System.err.println("Exception thrown when sending packet. Exiting.");
                System.err.println(exception);
            }
            println("New temperature timer period message sent");
            try {Thread.sleep(500);}
    		catch (InterruptedException exception) {}
    		requestData(sensor);
    }

    /**
     * Procedure to set battery timer period
     * @param baTi
     */
    public static void setBatTimer(int baTi){
            BatTimerMsg p_load = new BatTimerMsg();
            p_load.set_batTimerPeriod(baTi);
            payload = p_load;
            //Registers the message (false => Tx)
            SensorMessage msg_tx = new SensorMessage(p_load.toString(),false);
            Sensor sensor = getSensor(consola.getCurrentnodeId());
            sensor.AddMessage(msg_tx);
            //Send new battery timer period message
            try {
                    mif_tx.send(sensor.getNodeId(), payload);
                }
            catch (IOException exception) {
                System.err.println("Exception thrown when sending packet. Exiting.");
                System.err.println(exception);
            }
            println("New battery timer period message sent");
            try {Thread.sleep(500);}
    		catch (InterruptedException exception) {}
    		requestData(sensor);
    }

    /**
     * Procedure to set debug timer period
     * @param daTi
     */
    public static void setDebugTimer(int daTi){
            DataTimerMsg p_load = new DataTimerMsg();
            p_load.set_dataTimerPeriod(daTi);
            payload = p_load;
            //Registers the message (false => Tx)
            SensorMessage msg_tx = new SensorMessage(p_load.toString(),false);
            Sensor sensor = getSensor(consola.getCurrentnodeId());
            sensor.AddMessage(msg_tx);
            //Send new debug timer period message
            try {
                    mif_tx.send(sensor.getNodeId(), payload);
                }
            catch (IOException exception) {
                System.err.println("Exception thrown when sending packet. Exiting.");
                System.err.println(exception);
            }
            println("New debug timer period message sent");
            try {Thread.sleep(500);}
    		catch (InterruptedException exception) {}
    		requestData(sensor);
    }

    /**
     * Procedure to confirm alarm received
     * @param daTi
     */
    public static void confirmAlarm(){
            FireAlarmReceivedMsg p_load = new FireAlarmReceivedMsg();
            p_load.set_recAlarm((byte)1);
            payload = p_load;
            //Registers the message (false => Tx)
            SensorMessage msg_tx = new SensorMessage(p_load.toString(),false);
            Sensor sensor = getSensor(consola.getCurrentnodeId());
            sensor.AddMessage(msg_tx);
            //Send confirm alarm message
            try {
                    mif_tx.send(sensor.getNodeId(), payload);
                }
            catch (IOException exception) {
                System.err.println("Exception thrown when sending packet. Exiting.");
                System.err.println(exception);
            }
            println("Confirm alarm message sent");
    }
    
    private String toVolts(int counts, int vref){
    	DecimalFormat df = new DecimalFormat("#0.00");
        double d = (((double)counts)/1024)*((double)vref/1000)*2;
        return df.format(d);
    }
    
    private String toCentig(int counts, int vref){
    	DecimalFormat df = new DecimalFormat("#0.00");
        double voltsTemp = (((double)counts)/1024)*((double)vref/1000);
        double tempCentigrads = (voltsTemp-0.5)/0.01;
        return df.format(tempCentigrads);
    }
    
    private String toLux(int counts, int vref){
    	DecimalFormat df = new DecimalFormat("#0.00");
        double d = (15147*((double)vref/(double)counts))-4700;
        double lux = Math.pow(d/1000,(1/0.85));
        return df.format(lux);
    }
    
    /**
     * @return current time
     */
    public static String getCurrentTime() {
         Date d = Calendar.getInstance().getTime();
         return d.toGMTString();
    }
    
    public static void sensorDead(short nId){
    	Sensor s = getSensor(nId);
    	s.setDeadDate(getCurrentTime());
    	refreshConsole(s);
    	println("Sensor " + nId + " Mort");
    }
}
