#ifndef FIRE_SENSOR_H
#define FIRE_SENSOR_H

enum {
	TEMP,BAT,DATA,
	V_REF_MILIVOLTS		= 2560,
	TEMP_ALARM			= 320,		//Default temperature threshold
	BAT_LEVEL_ALARM		= 540,		//Default battery threshold
	N_TEMP				= 2000,		//Default temperature read Timer
	L_BAT				= 5000,		//Default battery read Timer
	M_DEBUG				= 0,	//Default data send timer
	AM_RSSIMSG 			= 10,
	AM_DATAMSG			= 40,
	AM_TEMPMSG			= 41,
	AM_BATMSG			= 42,
	AM_TEMPTHDMSG	 	= 43,
	AM_TEMPTIMERMSG		= 44,
	AM_BATTIMERMSG		= 45,
	AM_DATATIMERMSG		= 46,
	AM_FIREALARMRECEIVEDMSG	= 47,
	AM_FIREALARMMSG 	= 48,
	AM_BATALARMMSG		= 49,
	AM_FIREALARMACK		= 50,
	AM_STATEREQMSG		= 51,
	AM_STATEMSG			= 52,
	AM_DATAREQMSG		= 53
};

//defined active message structure to data report
typedef nx_struct DataMsg {
	nx_uint8_t moteId;
	nx_uint16_t countsTemp;
	nx_uint16_t countsPhoto;
	nx_uint16_t countsBat;
	nx_uint16_t vrefmilivolts;
	nx_uint16_t tempThd;
	nx_uint32_t tempTimer;
	nx_uint32_t batTimer;
	nx_uint32_t debugTimer;
} DataMsg;

//defined active message structure to temperature report
typedef nx_struct TempMsg {
	nx_uint8_t moteId;
	nx_uint16_t countsTemp;
	nx_uint16_t vrefmilivolts;
} TempMsg;

//defined active message structure to battery level report
typedef nx_struct BatMsg {
	nx_uint8_t moteId;
	nx_uint16_t countsBat;
	nx_uint16_t vrefmilivolts;
} BatMsg;

//defined active message structure to update threshold temperature
typedef nx_struct TempThdMsg {
	nx_uint16_t countsTemp;
}TempThdMsg;

//defined active message structure to update temperature timer period
typedef nx_struct TempTimerMsg {
	nx_uint32_t tempTimerPeriod;
}TempTimerMsg;

//defined active message structure to update battery timer period
typedef nx_struct BatTimerMsg {
	nx_uint32_t batTimerPeriod;
}BatTimerMsg;

//defined active message structure to update data timer period
typedef nx_struct DataTimerMsg {
	nx_uint32_t dataTimerPeriod;
}DataTimerMsg;

//defined active message structure to receive fire alarm confirmation
typedef nx_struct FireAlarmReceivedMsg {
	nx_bool recAlarm;
}FireAlarmReceivedMsg;

//defined active message structure to fire alarm
typedef nx_struct FireAlarmMsg {
	nx_uint8_t moteId;
	nx_bool manualAlarm;
	nx_uint16_t countsTemp;
	nx_uint16_t vrefmilivolts;
}FireAlarmMsg;

//defined active message structure to battery alarm
typedef nx_struct BatAlarmMsg {
	nx_uint8_t moteId;
	nx_uint16_t countsBat;
	nx_uint16_t vrefmilivolts;
}BatAlarmMsg;

//defined active message structure to receive fire alarm acknowledgment
typedef nx_struct FireAlarmAckMsg {
	nx_bool ackAlarm;
}FireAlarmAckMsg;

//defined active message structure to state request
typedef nx_struct StateReqMsg {
	nx_uint8_t moteId;
}StateReqMsg;


//defined active message structure to state update
typedef nx_struct StateMsg {
	nx_bool onStby;					// Update to state On (TRUE), hold Stby (FALSE)
	nx_uint16_t tempThd;			// temperature threshold
	nx_uint32_t tempTimerPeriod;	// temperature timer period
	nx_uint32_t batTimerPeriod;		// battery timer period
	nx_uint32_t debugTimerPeriod;	// data timer period
}StateMsg;

//defined active message structure to data request
typedef nx_struct DataReqMsg {
	nx_uint8_t seq;
}DataReqMsg;

//defined active message structure to RSSI
typedef nx_struct RssiMsg{
  nx_int16_t rssi;
} RssiMsg;

#endif /* FIRE_SENSOR_H */
